<?php
/**
 * @package            Joomla
 * @subpackage         Event Booking
 * @author             Tuan Pham Ngoc
 * @copyright          Copyright (C) 2010 - 2023 Ossolution Team
 * @license            GNU/GPL, see LICENSE.php
 */

use Joomla\CMS\Application\ApplicationHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Mail\MailHelper;
use Joomla\CMS\Table\Table;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\User\User;
use Joomla\Registry\Registry;
use Joomla\Utilities\ArrayHelper;
use MBWC\Component\MBWC\Site\Helper\MBWCHelper;
/**
 * EventbookingHelperOverrideMail class
 *
 * This class can be used to override some common methods used in EventbookingHellperMail class. It is needed when you need to
 * override these methods without having to worry about losing the customization while updating to future releases of Events Booking
 */
class EventbookingHelperOverrideMail extends EventbookingHelperMail
{

	/**
	 * Helper function for sending emails to registrants and administrator
	 *
	 * @param   EventbookingTableRegistrant  $row
	 * @param   RADConfig                    $config
	 */
	public static function sendEmails($row, $config)
	{
		$fieldSuffix = EventbookingHelper::getFieldSuffix($row->language);
		$event       = EventbookingHelperDatabase::getEvent($row->event_id, null, $fieldSuffix);
		$category    = EventbookingHelperDatabase::getCategory($event->main_category_id);

		EventbookingHelper::setEventMessagesDataFromCategory(
			$event,
			$category,
			['admin_email_body', 'user_email_body', 'user_email_body_offline', 'group_member_email_body'],
			$fieldSuffix
		);
		EventbookingHelper::setEventStringsDataFromCategory($event, $category, ['user_email_subject', 'notification_emails'], $fieldSuffix);

		if ($event->send_emails != -1)
		{
			$config->send_emails = $event->send_emails;
		}

		if ($config->send_emails == 3)
		{
			return;
		}

		$userId = $row->user_id ?: null;

		// Load frontend component language if needed
		EventbookingHelper::loadRegistrantLanguage($row);

		if (strlen(trim($event->notification_emails)) > 0)
		{
			$config->notification_emails = $event->notification_emails;
		}

		$mailer = static::getMailer($config, $event);

		if ($event->reply_to_email && MailHelper::isEmailAddress($event->reply_to_email))
		{
			$mailer->addReplyTo($event->reply_to_email);
		}
		elseif ($event->created_by && $config->send_email_to_event_creator)
		{
			$eventCreator = User::getInstance($event->created_by);

			if (MailHelper::isEmailAddress($eventCreator->email) && !$eventCreator->authorise('core.admin'))
			{
				$mailer->addReplyTo($eventCreator->email);
			}
		}

		if ($row->published == 3)
		{
			$typeOfRegistration = 2;
		}
		else
		{
			$typeOfRegistration = 1;
		}

		if ($config->multiple_booking)
		{
			$rowFields = EventbookingHelperRegistration::getFormFields($row->id, 4, $row->language, $userId, $typeOfRegistration);
		}
		elseif ($row->is_group_billing)
		{
			$rowFields = EventbookingHelperRegistration::getFormFields($row->event_id, 1, $row->language, $userId, $typeOfRegistration);
		}
		else
		{
			$rowFields = EventbookingHelperRegistration::getFormFields($row->event_id, 0, $row->language, $userId, $typeOfRegistration);
		}

		$form = new RADForm($rowFields);
		$data = EventbookingHelperRegistration::getRegistrantData($row, $rowFields);
		$form->bind($data);
		$form->buildFieldsDependency();

		$replaces = EventbookingHelper::callOverridableHelperMethod('Registration', 'buildTags', [$row, $form, $event, $config], 'Helper');

		// Get group members data
		$collectMemberInformation = EventbookingHelperRegistration::isCollectMembersInformation($event, $config);

		if ($row->is_group_billing && $collectMemberInformation)
		{
			$db    = Factory::getDbo();
			$query = $db->getQuery(true)
				->select('*')
				->from('#__eb_registrants')
				->where('group_id = ' . $row->id);
			$db->setQuery($query);
			$rowMembers = $db->loadObjectList();
		}
		else
		{
			$rowMembers = [];
		}

		$invoiceFilePath = '';

		if ($config->activate_invoice_feature
			&& $row->invoice_number
			&& ($config->send_invoice_to_admin || $config->send_invoice_to_customer))
		{
			$invoiceFilePath = EventbookingHelper::callOverridableHelperMethod('Helper', 'generateInvoicePDF', [$row]);

			// This is for backward-compatible only in case someone override generateInvoicePDF method
			if (!$invoiceFilePath)
			{
				$invoiceFilePath = JPATH_ROOT . '/media/com_eventbooking/invoices/' . EventbookingHelper::callOverridableHelperMethod(
						'Helper',
						'formatInvoiceNumber',
						[$row->invoice_number, $config, $row]
					) . '.pdf';
			}
		}

		// Send confirmation email to registrant
		if (in_array($config->send_emails, [0, 2]))
		{
			// Send to billing member, get attachments back and use it to add to attachments to group member email
			$attachments = EventbookingHelper::callOverridableHelperMethod(
				'Mail',
				'sendRegistrationEmailToRegistrant',
				[$mailer, $row, $rowMembers, $replaces, $rowFields, $invoiceFilePath]
			);

			// Send emails to group members
			if ($config->send_email_to_group_members && $row->is_group_billing && count($rowMembers))
			{
				static::sendRegistrationEmailToGroupMembers($mailer, $row, $rowMembers, $replaces, $attachments);
			}

			// Clear attachments
			$mailer->clearAttachments();
			$mailer->clearReplyTos();
		}

		// Send notification emails to admin if needed
		if (in_array($config->send_emails, [0, 1]))
		{
			static::sendRegistrationEmailToAdmin($mailer, $row, $form, $replaces, $rowFields, $invoiceFilePath);
		}
	}

	/**
	 * Send registration email to registrant
	 *
	 * @param   JMail                          $mailer
	 * @param   EventbookingTableRegistrant    $row
	 * @param   EventbookingTableRegistrant[]  $rowMembers
	 * @param   array                          $replaces
	 * @param   array                          $rowFields
	 * @param   string                         $invoiceFilePath
	 *
	 * @return array List of attachments which can be used for group members
	 */
	public static function sendRegistrationEmailToRegistrant($mailer, $row, $rowMembers, $replaces, $rowFields, $invoiceFilePath)
	{
		$config      = EventbookingHelper::getConfig();
	    $attachments=null;
        MBWCHelper::sendRegistrationEmails($row, $config);
		return $attachments;
	}

	/**
	 * Send registration email to group members
	 *
	 * @param   JMail                          $mailer
	 * @param   EventbookingTableRegistrant    $row
	 * @param   EventbookingTableRegistrant[]  $rowMembers
	 * @param   array                          $replaces
	 * @param   array                          $attachments
	 */
	protected static function sendRegistrationEmailToGroupMembers($mailer, $row, $rowMembers, $replaces, $attachments)
	{
		$config      = EventbookingHelper::getConfig();
	    $attachments=null;
        MBWCHelper::sendGroupRegistrationEmails($rowMembers, $config);
		return $attachments;
	}

	/**
	 * Send registration email to administrator
	 *
	 * @param   JMail                        $mailer
	 * @param   EventbookingTableRegistrant  $row
	 * @param   RADForm                      $form
	 * @param   array                        $replaces
	 * @param   array                        $rowFields
	 * @param   array                        $attachments
	 * @param   string                       $invoiceFilePath
	 */
	protected static function sendRegistrationEmailToAdmin($mailer, $row, $form, $replaces, $rowFields, $invoiceFilePath)
	{
		$config      = EventbookingHelper::getConfig();
	    $attachments=null;
        MBWCHelper::sendRegistrationLeaderEmails($row, $config);
		return $attachments;
	}
	
	public static function sendWaitinglistEmail($row, $config)
	{
		$config      = EventbookingHelper::getConfig();
	    $attachments=null;
        MBWCHelper::sendWaitinglistEmail($row, $config);
		return $attachments;
	}

}