<?php
/**
 * @package            Joomla
 * @subpackage         Event Booking
 * @author             Tuan Pham Ngoc
 * @copyright          Copyright (C) 2010 - 2023 Ossolution Team
 * @license            GNU/GPL, see LICENSE.php
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use MBWC\Component\MBWC\Site\Helper\MBWCHelper;
use Joomla\Registry\Registry;
use Joomla\CMS\Log\Log;
use Joomla\Utilities\ArrayHelper;

class EventbookingModelOverrideEvent extends EventbookingModelEvent
{

	/**
	 * Get all necessary data of an event
	 *
	 * @return mixed
	 */
	public function getEventData()
	{
		$config      = EventbookingHelper::getConfig();
		$db          = $this->getDbo();
		$query       = $db->getQuery(true);
		$fieldSuffix = EventbookingHelper::getFieldSuffix();
		$currentDate = $db->quote(EventbookingHelper::getServerTimeFromGMTTime());
		$query->select('a.*')
			->select("DATEDIFF(event_date, $currentDate) AS number_event_dates")
			->select("TIMESTAMPDIFF(MINUTE, a.late_fee_date, $currentDate) AS late_fee_date_diff")
			->select("TIMESTAMPDIFF(MINUTE, a.event_date, $currentDate) AS event_start_minutes")
			->select("TIMESTAMPDIFF(SECOND, registration_start_date, $currentDate) AS registration_start_minutes")
			->select("TIMESTAMPDIFF(MINUTE, cut_off_date, $currentDate) AS cut_off_minutes")
			->select("TIMESTAMPDIFF(MINUTE, $currentDate, early_bird_discount_date) AS date_diff")
			->select('(IFNULL(SUM(b.number_registrants), 0) + a.private_booking_count) AS total_registrants')
			->select($db->quoteName(['c.name' . $fieldSuffix, 'c.alias' . $fieldSuffix], ['location_name', 'location_alias']))
			->select('c.address AS location_address')
			->select('cat.tax_rate AS cat_tax_rate')
			->from('#__eb_events AS a')
			->leftJoin(
				'#__eb_registrants AS b ON (a.id = b.event_id AND b.group_id=0 AND (b.published in (1,4) OR (b.published = 0 AND b.payment_method LIKE "os_offline%")))'
			)
			->leftJoin('#__eb_locations AS c ON a.location_id = c.id ')
			->leftJoin('#__eb_categories AS cat ON a.main_category_id = cat.id')
			->where('a.id = ' . $this->state->id)
			->group('a.id');

		if ($config->show_event_creator)
		{
			$query->select('u.name AS creator_name')
				->leftJoin('#__users as u ON a.created_by = u.id');
		}

		if ($fieldSuffix)
		{
			static::handleMultilingualData($query, $fieldSuffix, 'a');
		}

		$db->setQuery($query);
		$row = $db->loadObject();

		// Get additional information about the event
		if ($row)
		{
			if (!EventbookingHelper::isValidMessage($row->description))
			{
				$row->description = $row->short_description;
			}

			if ($row->image)
			{
				$row->image = EventbookingHelperHtml::getCleanImagePath($row->image);
			}

			$row->tax_rate = EventbookingHelperRegistration::calculateEventTaxRate($row->id);

			EventbookingHelper::callOverridableHelperMethod('Data', 'preProcessEventData', [[$row], 'item']);
		}

		return $row;
	}
	
	public function validateFormInput($input)
	{
	    $errors=parent::validateFormInput($input);

        if ($input->getInt('published', 0))
        {
            $error=MBWCHelper::checkDuplicateLeader($input, 0);
            if ($error)
            {
                $errors[]=$error;
            }
            $error=MBWCHelper::checkDuplicateRegistrant($input, 0);
            if ($error)
            {
                $errors[]=$error;
            }
        }

		$error = MBWCHelper::hasCategoryPermissions($input);
		if ($error)
		{
            $errors[]=$error;
		}
	    return $errors;
	}
	protected function beforeStore($row, $input, $isNew)
	{
		$app    = Factory::getApplication();
		$user   = Factory::getUser();
		$config = EventbookingHelper::getConfig();
		$data = $input->getData();
        $params=$data['params'];
        $petrol_cost=$params['field_petrol_cost'];
        $field_address=$params['field_address'];
        $address1=trim(rawurldecode($field_address),' ');
        
        if ($address1)
        {
            // if we match lat long already, don't need to do anything
            if (preg_match_all( "/[0-9]/", $address1 ) >= 5 && preg_match_all( "/[a-z|A-Z]/", $address1 ) <= 5)
            {
                $address1=str_replace('%2C',',',str_replace('%20', '',$address1));
                $polyArray1=explode(',',$address1);
                // average opposite sides
                $myadd1=str_replace(' ', '',$polyArray1[1]).','.str_replace(' ', '',$polyArray1[0]);
                $params['field_address_long']=str_replace(' ', '',$polyArray1[1]);
                $params['field_address_lat']=str_replace(' ', '',$polyArray1[0]);
            }
            else
            {
                $melway=0;
                $matches=null;
                $resp=preg_match('/^[ ]*Mel[.: ]+(.*)$/',$address1,$matches);
                if ($resp)
                {
                    $address1=$matches[1];
                }
                $response = @file_get_contents('https://online.melway.com.au/melway/php/MelwaySearchInfo.php?directory=Mel&search='.rawurlencode($address1),0, stream_context_create(["http"=>["timeout"=>10]]));
                if ($response !== false && strpos($response,"syntax error") === FALSE)
                {
                    $response = new SimpleXMLElement($response);
                    if ($response->marker[1]['COL3'] == $address1)
                    {
                        $poly=substr($response->marker[1]['COL4'],15,-3);
                        $polyArray=explode(',',$poly);
                        // average opposite sides
                        $poly1=explode(' ',$polyArray[0]);
                        $poly2=explode(' ',$polyArray[2]);

                        $params['field_address_long']=str_replace(' ', '',($poly1[0] + $poly2[0])/2);
                        $params['field_address_lat']=str_replace(' ', '',($poly1[1] + $poly2[1])/2);
                        $melway=1;
                    }
                }
                if (!$melway)
                {
                    $url='https://api.openrouteservice.org/geocode/search?api_key=5b3ce3597851110001cf6248634bfcb8521544d58f6b7c9cbf9a1eaf&text='.rawurlencode($address1);
                    $response = @file_get_contents($url,0, stream_context_create(["http"=>["timeout"=>10]]));
                    if ($response !== false  && strpos($response,"error") === FALSE)
                    {
                        $obj=json_decode($response);
                        $params['field_address_long']=($obj->bbox[0] + $obj->bbox[2])/2;
                        $params['field_address_lat']=($obj->bbox[1]+$obj->bbox[3])/2;
                    }
                    else
                    {
                        $params['field_address_lat']="";
                        $params['field_address_long']="";
                    }
                }
            }
        }
        
        $field_address_start=$params['field_address_start'];
        $address2=trim(rawurldecode($field_address_start),' ');
        if ($address2)
        {
            if (preg_match_all( "/[0-9]/", $address2 ) >= 5 && preg_match_all( "/[a-z|A-Z]/", $address2 ) <= 5)
            {
                $address2=str_replace('%2C',',',str_replace('%20', '',$address2));
                $polyArray2=explode(',',$address2);
                // average opposite sides
                $params['field_address_start_long']=str_replace(' ', '',$polyArray2[1]);
                $params['field_address_start_lat']=str_replace(' ', '',$polyArray2[0]);
            }
            else
            {
                $melway=0;
                $matches=null;
                $resp=preg_match('/^[ ]*Mel[.: ]+(.*)$/',$address2,$matches);
                if ($resp)
                {
                    $address2=$matches[1];
                }
                $response = @file_get_contents('https://online.melway.com.au/melway/php/MelwaySearchInfo.php?directory=Mel&search='.rawurlencode($address2),0, stream_context_create(["http"=>["timeout"=>10]]));
                if ($response !== false && strpos($response,"syntax error") === FALSE)
                {
                    $response = new SimpleXMLElement($response);
                    if ($response->marker[1]['COL3'] ==$address2)
                    {
                        $melway=1;
                        $poly=substr($response->marker[1]['COL4'],15,-3);
                        $polyArray=explode(',',$poly);
                        // average opposite sides
                        $poly1=explode(' ',$polyArray[0]);
                        $poly2=explode(' ',$polyArray[2]);
                        $params['field_address_start_long']=str_replace(' ', '',($poly1[0] + $poly2[0])/2);
                        $params['field_address_start_lat']=str_replace(' ', '',($poly1[1] + $poly2[1])/2);
                     }
                }
                if (!$melway)
                {
                    $url='https://api.openrouteservice.org/geocode/search?api_key=5b3ce3597851110001cf6248634bfcb8521544d58f6b7c9cbf9a1eaf&text='.rawurlencode($address2);
                    $response = @file_get_contents($url,0, stream_context_create(["http"=>["timeout"=>10]]));
                    if ($response !== false && strpos($response,"error") === FALSE)
                    {
                        $obj=json_decode($response);
                        $params['field_address_start_long']=($obj->bbox[0] + $obj->bbox[2])/2;
                        $params['field_address_start_lat']=($obj->bbox[1]+$obj->bbox[3])/2;
                    }
                    else
                    {
                        $params['field_address_start_lat']="";
                        $params['field_address_start_long']="";
                    }
                }
            }
        }
        if ($params['field_address_start_lat'] && $params['field_address_lat'])
        {
            $myadd1=$params['field_address_long'].",".$params['field_address_lat'];
            $myadd2=$params['field_address_start_long'].",".$params['field_address_start_lat'];
            $url='https://api.openrouteservice.org/v2/directions/driving-car?api_key=5b3ce3597851110001cf6248634bfcb8521544d58f6b7c9cbf9a1eaf&start='.$myadd1.'&end='.$myadd2;
            $response = @file_get_contents($url,0, stream_context_create(["http"=>["timeout"=>10]]));
            if ($response !== false && strpos($response,"error") === FALSE)
            {
                $obj=json_decode($response);
 
                if ($obj->features[0]->properties->summary && property_exists($obj->features[0]->properties->summary,'distance'))
                {
                    $params['driving_distance']=round($obj->features[0]->properties->summary->distance/1000);
                }
                if ($obj->features[0]->properties->summary && property_exists($obj->features[0]->properties->summary,'duration'))
                {
                  $params['driving_duration']=round($obj->features[0]->properties->summary->duration/60);
                }
            }
            if ($petrol_cost)
            {
                $params['carpool_cost']=round(($params['driving_distance']*2*11.11*$petrol_cost/100+$params['driving_distance']*2*.1)/3);
            }
        }
 		// Set $data array back to input object
        $data['params']=$params;
		$input->setData($data);

		parent::beforeStore($row, $input, $isNew);
	}
	
	protected function afterStore($row, $input, $isNew)
	{
		parent::afterStore($row, $input, $isNew);

		$app  = Factory::getApplication();
		$user = Factory::getUser();
        $fieldextras = new Registry($row->custom_fields);
		$db = Factory::getDbo();

        if ($row->registration_type != 4)
        {
            $userexists=false;
            if ($fieldextras->exists('field_leader_id1') && $fieldextras->get('field_leader_id1'))
            {
                $sql="select distinct 1 from jos_eb_registrants as b join jos_eb_events as c on c.id=b.event_id where b.user_id=".$fieldextras->get('field_leader_id1')." and (b.published=1 or b.published=3) and b.is_group_billing=0 and c.id=".$row->id;
                $db->setQuery($sql);
                $userexists = $db->loadResult();
                if (!$userexists)
                {
	                MBWCHelper::addUser($row->id, $fieldextras->get('field_leader_id1'), false, false);
                }
            }
            $userexists=false;
            if ($fieldextras->exists('field_leader_id2') && $fieldextras->get('field_leader_id2'))
            {
                $sql="select distinct 1 from jos_eb_registrants as b join jos_eb_events as c on c.id=b.event_id where b.user_id=".$fieldextras->get('field_leader_id2')." and (b.published=1 or b.published=3) and b.is_group_billing=0 and c.id=".$row->id;
                $db->setQuery($sql);
                $userexists = $db->loadResult();
	            if (!$userexists)
	            {
	                MBWCHelper::addUser($row->id, $fieldextras->get('field_leader_id2'), false, false);
	            }
            }
            $userexists=false;
            if ($fieldextras->exists('field_leader_id3') && $fieldextras->get('field_leader_id3'))
            {
                $sql="select distinct 1 from jos_eb_registrants as b join jos_eb_events as c on c.id=b.event_id where b.user_id=".$fieldextras->get('field_leader_id3')." and (b.published=1 or b.published=3) and b.is_group_billing=0 and c.id=".$row->id;
                $db->setQuery($sql);
                $userexists = $db->loadResult();
	            if (!$userexists)
	            {
	                MBWCHelper::addUser($row->id, $fieldextras->get('field_leader_id3'), false, false);
	            }
            }
        }
 
	}

}
